#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
from __future__ import division, with_statement
"""
This one will parse my honors.org work log and try to figure out iOS dev timestamps for putting into the nickwinter.net percentile feedback zone.

Or, if passed "CoCo" as argv[1], it will upload for that project instead.
"""

import os
import sys
import re
import collections
import random
import cPickle as pickle
import urllib
from datetime import datetime, date, timedelta

cat = sys.argv[1] if len(sys.argv) > 1 else ['iPhone app', 'CoCo', 'remnants'][-1]

honors_path = os.path.expanduser(os.path.join('~', 'Dropbox', 'org', 'honors.org'))

def ttos(td):
    return td.seconds + (td.days*86400)

def dtodt(d):
    if isinstance(d, datetime):
        d = d.date()
    return datetime(d.year, d.month, d.day)

def epoch(dt):
    if not dt:
        return 0
    if not isinstance(dt, datetime):
        dt = datetime(dt.year, dt.month, dt.day)
    return time.mktime(dt.utctimetuple()) + 0.000001 * dt.microsecond

def fromEpoch(seconds):
    return datetime.utcfromtimestamp(seconds)

def parse_timestamp(work_line):
    """ "<2012-01-29 Sun 17:29> 3.45 hours" --> datetime"""
    s = re.sub('>.*', '', work_line[1:])
    try:
        dt = datetime.strptime(s, "%Y-%m-%d %a %H:%M")
    except ValueError:
        dt = datetime.strptime(s, "%Y-%m-%d %a")
    return dt


def main():
    with open(honors_path, 'rb') as f:
        lines = f.read().decode("utf-8").split('\n')
    four = timedelta(hours=4)
    cat_work_days = collections.defaultdict(list)
    cat_work_categories = collections.defaultdict(float)
    if cat == 'iPhone app':
        proj_start = date(2010, 11, 5)
        proj_end = date(2012, 9, 1)
    elif cat == 'remnants':
        proj_start = date(2012, 9, 1)
        proj_end = date(2013, 1, 1)
    elif cat == 'CoCo':
        proj_start = date(2013, 1, 8)
        proj_end = date(2017, 1, 1)
    working = False
    interested = False
    category = None
    for i, line in enumerate(lines):
        if line.startswith('<20'):
            timestamp = parse_timestamp(line)
            if timestamp < dtodt(proj_start):
                continue  # hadn't started project yet
            if working:
                if interested:
                    start_ofs = working - four - dtodt(working - four)
                    end_ofs = timestamp - four - dtodt(timestamp - four)
                    cat_work_days[(working - four).date()
                                  ].append([ttos(start_ofs), ttos(end_ofs)])
                if interested and (working - four).date() >= proj_start and \
                        (working - four).date() < proj_end:
                    cat_work_categories[category] += \
                        (ttos(end_ofs) - ttos(start_ofs)) / 3600
                    #print "Just assigned %.2f to %s for %s"%(
                    #    (ttos(end_ofs) - ttos(start_ofs)) / 3600, category,
                    #    '\n'.join(lines[i-2:i+1]))
                    #print '\n'.join(lines[i-2:i+1])
                working = False
                interested = False
                category = None
            else:
                working = timestamp
        if line.startswith('%s ('%cat):
            if working:
                interested = True
                category = re.sub(r'%s \((.*?)\).*'%cat, r'\1', line)
            else:
                print "Hmm; got line %d: %s while not working?"%(i, line)
        elif '%s ('%cat in line:
            h1 = int(line.split(':')[0])
            m1 = int(line.split(':')[1].split(' - ')[0])
            h2 = int(line.split(' - ')[1].split(':')[0])
            m2 = int(line.split(' - ')[1].split(':')[1])
            t1 = 60 * h1 + m1
            t2 = 60 * h2 + m2
            if t1 < 4 * 60:
                t1 += 20 * 60
            else:
                t1 -= 4 * 60
            if t2 < 4 * 60:
                t2 += 20 * 60
            else:
                t2 -= 4 * 60
            cat_work_days[(working - four).date()].append([t1 * 60, t2 * 60])
            
            if (working - four).date() >= proj_start and \
                    (working - four).date() < proj_end:
                category = re.sub(r'.*%s \((.*?)\).*'%cat, r'\1', line)
                cat_work_categories[category] += (t2 - t1) / 60
                #print "Just assigned %.2f to %s for %s"%(
                #    (t2 - t1) / 60, category, line)
                #print line
        elif line.startswith('%s: <'%cat):
            ts1 = parse_timestamp(re.sub(r".*?(<.*?>).*", r"\1", line))
            if ts1 < dtodt(proj_start):
                continue  # hadn't started project yet
            ts2, working = None, True
            if line.count('<') > 1:
                ts2 = parse_timestamp(re.sub(r".*? - (<.*?>).*", r"\1", line))
                working = False
            else:
                ts2 = datetime.now()
            start_ofs = ts1 - four - dtodt(ts1 - four)
            end_ofs = ts2 - four - dtodt(ts2 - four)
            cat_work_day = cat_work_days[(ts2 - four).date()]
            cat_work_day.append([ttos(start_ofs)])
            if not working:
                cat_work_day[-1].append(ttos(end_ofs))
            cat_work_categories[category or "work"] += \
                (ttos(end_ofs) - ttos(start_ofs)) / 3600

    total_since_last_count = sum(cat_work_categories.values())
    total_today, total_this_week = 0, 0
    today = (datetime.now() - four).date()
    for i in range(7):
        past_date = today - timedelta(days=i)
        if past_date.weekday() > today.weekday(): continue
        for work_pair in cat_work_days[past_date]:
            start = work_pair[0]
            stop = work_pair[1] if len(work_pair) > 1 else ttos(
                datetime.now() - four - dtodt(datetime.now() - four))
            hours = (stop - start) / 3600
            total_this_week += hours
            if i == 0:
                total_today += hours
    if len(cat_work_categories) > 1:
        print '\n'.join(['%s:\t%.2f'%x for x in cat_work_categories.items()])
        print '-'*50
    print "%s\tToday: %.2f\tWeek: %.2f\tTotal since %s: %.2f"%(
        cat, total_today, total_this_week, proj_start, total_since_last_count)
    if False:
        for day, times in sorted(cat_work_days.items(), key=lambda x: x[0]):
            if len(times) < 2: continue
            print "%s %.2f"%(day, sum([(t2 - t1) / 3600 for (t1, t2) in times]))

    if cat == 'CoCo':
        wrs = []
        for d, times in sorted(cat_work_days.items(), key=lambda x: x[0]):
            if (date.today() - d).days > 1:
                continue   # don't need to upload old ones
            work_starts = [float(x[0]) for x in times]
            work_stops = [float(x[1]) for x in times if len(x) > 1]
            wrs.append([d, cat, cat, "work", work_starts, work_stops])
        if wrs:
            s = pickle.dumps(wrs)
            data = urllib.urlencode({'work_records': s})
            host = ['http://www.nickwinter.net', 'http://localhost:9001'][0]
            urllib.urlopen('%s/work/update'%host, data)

    elif cat == 'iPhone':
        return
        print 'wrs = []'
        for d, times in sorted(cat_work_days.items(), key=lambda x: x[0]):
            if d >= date(2012, 1, 18):
                continue   # have these in the system already, with more precision
            work_starts = [float(x[0]) for x in times]
            work_stops = [float(x[1]) for x in times]
            print ('wrs.append(models.WorkRecord(date="%s", category="Skritter", '
                   'project="%s", task="work", start_times=%s, '
                   'stop_times=%s))'%(d, cat, work_starts, work_stops))
                          

if __name__ == "__main__":
    main()
